import contextlib
import io
import os
import random
import re
import socket
import sys

import pytest

PASSWORD_REGEX = re.compile(r"^([a-z]+)(-[a-z]+)?-([a-z]+)(-[a-z]+)?$")


# TODO: Improve the Common tests to test it all as a single class


class TestBuildPassword:
    @pytest.mark.parametrize(
        "test_input,expected",
        (
            # VALID, two lowercase words, separated by a hyphen
            ("syrup-enzyme", True),
            ("caution-friday", True),
            # VALID, two lowercase words, with one hyphenated compound word
            ("drop-down-thimble", True),
            ("unmixed-yo-yo", True),
            # VALID, two lowercase hyphenated compound words, separated by hyphen
            ("yo-yo-drop-down", True),
            ("felt-tip-t-shirt", True),
            ("hello-world", True),
            # INVALID
            ("Upper-Case", False),
            ("digits-123", False),
            ("too-many-hyphens-", False),
            ("symbols-!@#$%", False),
        ),
    )
    def test_build_password_regex(self, test_input, expected):
        """Test that `PASSWORD_REGEX` accounts for the following patterns

        There are a few hyphenated words in `wordlist.txt`:
            * drop-down
            * felt-tip
            * t-shirt
            * yo-yo

        These words cause a few extra potential password patterns:
            * word-word
            * hyphenated-word-word
            * word-hyphenated-word
            * hyphenated-word-hyphenated-word
        """

        assert bool(PASSWORD_REGEX.match(test_input)) == expected

    def test_build_password_unique(self, common_obj, sys_onionshare_dev_mode):
        assert common_obj.build_password() != common_obj.build_password()


class TestDirSize:
    def test_temp_dir_size(self, common_obj, temp_dir_1024_delete):
        """dir_size() should return the total size (in bytes) of all files
        in a particular directory.
        """

        assert common_obj.dir_size(temp_dir_1024_delete) == 1024


class TestEstimatedTimeRemaining:
    @pytest.mark.parametrize(
        "test_input,expected",
        (
            ((2, 676, 12), "8h14m16s"),
            ((14, 1049, 30), "1h26m15s"),
            ((21, 450, 1), "33m42s"),
            ((31, 1115, 80), "11m39s"),
            ((336, 989, 32), "2m12s"),
            ((603, 949, 38), "36s"),
            ((971, 1009, 83), "1s"),
        ),
    )
    def test_estimated_time_remaining(
        self, common_obj, test_input, expected, time_time_100
    ):
        assert common_obj.estimated_time_remaining(*test_input) == expected

    @pytest.mark.parametrize(
        "test_input",
        (
            (10, 20, 100),  # if `time_elapsed == 0`
            (0, 37, 99),  # if `download_rate == 0`
        ),
    )
    def test_raises_zero_division_error(self, common_obj, test_input, time_time_100):
        with pytest.raises(ZeroDivisionError):
            common_obj.estimated_time_remaining(*test_input)


class TestFormatSeconds:
    @pytest.mark.parametrize(
        "test_input,expected",
        (
            (0, "0s"),
            (26, "26s"),
            (60, "1m"),
            (947.35, "15m47s"),
            (1847, "30m47s"),
            (2193.94, "36m34s"),
            (3600, "1h"),
            (13426.83, "3h43m47s"),
            (16293, "4h31m33s"),
            (18392.14, "5h6m32s"),
            (86400, "1d"),
            (129674, "1d12h1m14s"),
            (56404.12, "15h40m4s"),
        ),
    )
    def test_format_seconds(self, common_obj, test_input, expected):
        assert common_obj.format_seconds(test_input) == expected

    # TODO: test negative numbers?
    @pytest.mark.parametrize("test_input", ("string", lambda: None, [], {}, set()))
    def test_invalid_input_types(self, common_obj, test_input):
        with pytest.raises(TypeError):
            common_obj.format_seconds(test_input)


class TestGetAvailablePort:
    @pytest.mark.parametrize(
        "port_min,port_max",
        ((random.randint(1024, 1500), random.randint(1800, 2048)) for _ in range(50)),
    )
    def test_returns_an_open_port(self, common_obj, port_min, port_max):
        """get_available_port() should return an open port within the range"""

        port = common_obj.get_available_port(port_min, port_max)
        assert port_min <= port <= port_max
        with socket.socket() as tmpsock:
            tmpsock.bind(("127.0.0.1", port))


class TestGetPlatform:
    def test_darwin(self, platform_darwin, common_obj):
        assert common_obj.platform == "Darwin"

    def test_linux(self, platform_linux, common_obj):
        assert common_obj.platform == "Linux"

    def test_windows(self, platform_windows, common_obj):
        assert common_obj.platform == "Windows"


class TestGetTorPaths:
    @pytest.mark.skipif(sys.platform != "darwin", reason="requires MacOS")
    def test_get_tor_paths_darwin(
        self, platform_darwin, common_obj, sys_frozen, sys_meipass
    ):
        base_path = os.path.dirname(
            os.path.dirname(os.path.dirname(common_obj.get_resource_path("")))
        )
        tor_path = os.path.join(base_path, "Resources", "Tor", "tor")
        tor_geo_ip_file_path = os.path.join(base_path, "Resources", "Tor", "geoip")
        tor_geo_ipv6_file_path = os.path.join(base_path, "Resources", "Tor", "geoip6")
        obfs4proxy_file_path = os.path.join(base_path, "Resources", "Tor", "obfs4proxy")
        meek_client_file_path = os.path.join(
            base_path, "Resources", "Tor", "meek-client"
        )
        snowflake_file_path = os.path.join(
            base_path, "Resources", "Tor", "snowflake-client"
        )
        assert common_obj.get_tor_paths() == (
            tor_path,
            tor_geo_ip_file_path,
            tor_geo_ipv6_file_path,
            obfs4proxy_file_path,
            snowflake_file_path,
            meek_client_file_path,
        )

    @pytest.mark.skipif(sys.platform != "linux", reason="requires Linux")
    def test_get_tor_paths_linux(self, platform_linux, common_obj):
        (
            tor_path,
            tor_geo_ip_file_path,
            tor_geo_ipv6_file_path,
            _,  # obfs4proxy is optional
            _,  # snowflake-client is optional
            _,  # meek-client is optional
        ) = common_obj.get_tor_paths()

        assert os.path.basename(tor_path) == "tor"
        assert (
            tor_geo_ip_file_path == "/usr/share/tor/geoip"
            or tor_geo_ip_file_path == "/usr/local/share/tor/geoip"
        )
        assert (
            tor_geo_ipv6_file_path == "/usr/share/tor/geoip6"
            or tor_geo_ipv6_file_path == "/usr/local/share/tor/geoip6"
        )

    @pytest.mark.skipif(sys.platform != "win32", reason="requires Windows")
    def test_get_tor_paths_windows(self, platform_windows, common_obj, sys_frozen):
        base_path = os.path.join(
            os.path.dirname(os.path.dirname(common_obj.get_resource_path(""))),
            "resources",
            "tor",
        )
        tor_path = os.path.join(os.path.join(base_path, "Tor"), "tor.exe")
        obfs4proxy_file_path = os.path.join(
            os.path.join(base_path, "Tor"), "obfs4proxy.exe"
        )
        snowflake_file_path = os.path.join(
            os.path.join(base_path, "Tor"), "snowflake-client.exe"
        )
        meek_client_file_path = os.path.join(
            os.path.join(base_path, "Tor"), "meek-client.exe"
        )
        tor_geo_ip_file_path = os.path.join(
            os.path.join(os.path.join(base_path, "Data"), "Tor"), "geoip"
        )
        tor_geo_ipv6_file_path = os.path.join(
            os.path.join(os.path.join(base_path, "Data"), "Tor"), "geoip6"
        )
        assert common_obj.get_tor_paths() == (
            tor_path,
            tor_geo_ip_file_path,
            tor_geo_ipv6_file_path,
            obfs4proxy_file_path,
            snowflake_file_path,
            meek_client_file_path,
        )


class TestHumanReadableFilesize:
    @pytest.mark.parametrize(
        "test_input,expected",
        (
            (1024 ** 0, "1.0 B"),
            (1024 ** 1, "1.0 KiB"),
            (1024 ** 2, "1.0 MiB"),
            (1024 ** 3, "1.0 GiB"),
            (1024 ** 4, "1.0 TiB"),
            (1024 ** 5, "1.0 PiB"),
            (1024 ** 6, "1.0 EiB"),
            (1024 ** 7, "1.0 ZiB"),
            (1024 ** 8, "1.0 YiB"),
        ),
    )
    def test_human_readable_filesize(self, common_obj, test_input, expected):
        assert common_obj.human_readable_filesize(test_input) == expected


class TestLog:
    def test_output(self, common_obj, time_strftime):
        common_obj.verbose = True

        # From: https://stackoverflow.com/questions/1218933
        with io.StringIO() as buf, contextlib.redirect_stdout(buf):
            common_obj.log("TestModule", "dummy_func")
            common_obj.log("TestModule", "dummy_func", "TEST_MSG")
            output = buf.getvalue()

        line_one, line_two, _ = output.split("\n")
        assert (
            "[Jun 06 2013 11:05:00]" in line_one and "TestModule.dummy_func" in line_one
        )
        assert (
            "[Jun 06 2013 11:05:00]" in line_two
            and "TestModule.dummy_func" in line_two
            and "TEST_MSG" in line_two
        )



import os

import pytest

from onionshare_cli import OnionShare
from onionshare_cli.common import Common
from onionshare_cli.mode_settings import ModeSettings


class MyOnion:
    def __init__(self):
        self.auth_string = "TestHidServAuth"
        self.private_key = ""
        self.scheduled_key = None

    @staticmethod
    def start_onion_service(
        self, mode, mode_settings_obj, await_publication=True, save_scheduled_key=False
    ):
        return "test_service_id.onion"


@pytest.fixture
def onionshare_obj():
    common = Common()
    return OnionShare(common, MyOnion())


@pytest.fixture
def mode_settings_obj():
    common = Common()
    return ModeSettings(common)


class TestOnionShare:
    def test_init(self, onionshare_obj):
        assert onionshare_obj.hidserv_dir is None
        assert onionshare_obj.onion_host is None
        assert onionshare_obj.local_only is False

    def test_start_onion_service(self, onionshare_obj, mode_settings_obj):
        onionshare_obj.start_onion_service("share", mode_settings_obj)
        assert 17600 <= onionshare_obj.port <= 17650
        assert onionshare_obj.onion_host == "test_service_id.onion"

    def test_start_onion_service_local_only(self, onionshare_obj, mode_settings_obj):
        onionshare_obj.local_only = True
        onionshare_obj.start_onion_service("share", mode_settings_obj)
        assert onionshare_obj.onion_host == "127.0.0.1:{}".format(onionshare_obj.port)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
OnionShare | https://onionshare.org/

Copyright (C) 2014-2022 Micah Lee, et al. <micah@micahflee.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
import setuptools

with open(os.path.join("onionshare_cli", "resources", "version.txt")) as f:
    version = f.read().strip()

setuptools.setup(
    name="onionshare-cli",
    version=version,
    long_description=(
        "OnionShare lets you securely and anonymously send and receive files. It works by starting a web server, "
        "making it accessible as a Tor onion service, and generating an unguessable web address so others can "
        "download files from you, or upload files to you. It does _not_ require setting up a separate server or "
        "using a third party file-sharing service."
    ),
    author="Micah Lee",
    author_email="micah@micahflee.com",
    maintainer="Micah Lee",
    maintainer_email="micah@micahflee.com",
    url="https://onionshare.org",
    license="GPLv3",
    keywords="onion, share, onionshare, tor, anonymous, web server",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Framework :: Flask",
        "Topic :: Communications :: File Sharing",
        "Topic :: Security :: Cryptography",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Intended Audience :: End Users/Desktop",
        "Operating System :: OS Independent",
        "Environment :: Web Environment",
    ],
    packages=[
        "onionshare_cli",
        "onionshare_cli.web",
    ],
    package_data={
        "onionshare_cli": [
            "resources/*",
            "resources/static/*",
            "resources/static/css/*",
            "resources/static/img/*",
            "resources/static/js/*",
            "resources/templates/*",
        ]
    },
    entry_points={
        "console_scripts": [
            "onionshare-cli = onionshare_cli:main",
        ],
    },
)



#!/usr/bin/env python3
import os
import inspect
import click
import platform
import subprocess
import shutil
import glob
import itertools

from common import get_binary_arches

root = os.path.dirname(
    os.path.dirname(
        os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
    )
)
desktop_dir = os.path.join(root, "desktop")

identity_name_application = "Developer ID Application: Lockdown Systems LLC (G762K6CH36)"
entitlements_plist_path = f"{desktop_dir}/package/Entitlements.plist"


def get_app_path():
    return os.path.join(desktop_dir, "build", "OnionShare.app")


def run(cmd, cwd=None, error_ok=False):
    print(f"{cmd} # cwd={cwd}")
    subprocess.run(cmd, cwd=cwd, check=True)


def get_size(dir):
    size = 0
    for path, dirs, files in os.walk(dir):
        for f in files:
            fp = os.path.join(path, f)
            size += os.path.getsize(fp)
    return size


def sign(path, entitlements, identity):
    run(
        [
            "codesign",
            "--sign",
            identity,
            "--entitlements",
            str(entitlements),
            "--timestamp",
            "--deep",
            "--force",
            "--options",
            "runtime,library",
            str(path),
        ]
    )


def get_binaries():
    pass


@click.group()
def main():
    """
    macOS build tasks
    """


@main.command()
def cleanup_build():
    """Delete unused PySide6 stuff to save space"""
    app_path = get_app_path()
    before_size = get_size(app_path)

    print("> Delete unused Qt Frameworks")
    for framework in [
        "QtMultimediaQuick",
        "QtQuickControls2",
        "QtQuickParticles",
        "QtRemoteObjects",
        "Qt3DInput",
        "QtNetworkAuth",
        "QtDataVisualization",
        "QtWebEngineCore",
        "Qt3DQuickRender",
        "Qt3DQuickExtras",
        "QtDesigner",
        "QtNfc",
        "QtQuick3DAssetImport",
        "QtWebEngineWidgets",
        "QtQuickWidgets",
        "Qt3DQuickInput",
        "Qt3DQuickScene2D",
        "Qt3DRender",
        "QtQuick3DRuntimeRender",
        "QtHelp",
        "QtPrintSupport",
        "QtCharts",
        "QtWebSockets",
        "QtQuick3DUtils",
        "QtQuickTemplates2",
        "QtPositioningQuick",
        "Qt3DCore",
        "QtXml",
        "QtSerialPort",
        "QtQuick",
        "QtScxml",
        "QtQml",
        "Qt3DExtras",
        "QtWebChannel",
        "QtMultimedia",
        "QtQmlWorkerScript",
        "QtVirtualKeyboard",
        "QtOpenGL",
        "Qt3DQuick",
        "QtTest",
        "QtPositioning",
        "QtBluetooth",
        "QtQuick3D",
        "Qt3DLogic",
        "QtQuickShapes",
        "QtQuickTest",
        "QtNetwork",
        "QtDesignerComponents",
        "QtMultimediaWidgets",
        "QtQmlModels",
        "Qt3DQuickAnimation",
        "QtSensors",
        "Qt3DAnimation",
        "QtSql",
        "QtConcurrent",
        "QtChartsQml",
        "QtDataVisualizationQml",
        "QtLabsAnimation",
        "QtLabsFolderListModel",
        "QtLabsQmlModels",
        "QtLabsSettings",
        "QtLabsSharedImage",
        "QtLabsWavefrontMesh",
        "QtOpenGLWidgets",
        "QtQmlCore",
        "QtQmlLocalStorage",
        "QtQmlXmlListModel",
        "QtQuick3DAssetUtils",
        "QtQuick3DEffects",
        "QtQuick3DGlslParser",
        "QtQuick3DHelpers",
        "QtQuick3DIblBaker",
        "QtQuick3DParticleEffects",
        "QtQuick3DParticles",
        "QtQuickControls2Impl",
        "QtQuickDialogs2",
        "QtQuickDialogs2QuickImpl",
        "QtQuickDialogs2Utils",
        "QtQuickLayouts",
        "QtQuickTimeline",
        "QtRemoteObjectsQml",
        "QtScxmlQml",
        "QtSensorsQuick",
        "QtShaderTools",
        "QtStateMachine",
        "QtStateMachineQml",
        "QtSvgWidgets",
        "QtUiTools",
        "QtWebEngineQuick",
        "QtWebEngineQuickDelegatesQml",
    ]:
        try:
            shutil.rmtree(
                f"{app_path}/Contents/MacOS/lib/PySide6/Qt/lib/{framework}.framework"
            )
            print(
                f"Deleted: {app_path}/Contents/MacOS/lib/PySide6/Qt/lib/{framework}.framework"
            )
        except FileNotFoundError:
            pass
        try:
            os.remove(f"{app_path}/Contents/MacOS/lib/PySide6/{framework}.abi3.so")
            print(f"Deleted: {app_path}/Contents/MacOS/lib/PySide6/{framework}.abi3.so")
        except FileNotFoundError:
            pass
        try:
            os.remove(f"{app_path}/Contents/MacOS/lib/PySide6/{framework}.pyi")
            print(f"Deleted: {app_path}/Contents/MacOS/lib/PySide6/{framework}.pyi")
        except FileNotFoundError:
            pass

    print("> Delete more unused PySide6 stuff to save space")
    for filename in [
        f"{app_path}/Contents/Resources/lib/PySide6/Designer.app",
        f"{app_path}/Contents/Resources/lib/PySide6/glue",
        f"{app_path}/Contents/Resources/lib/PySide6/include",
        f"{app_path}/Contents/Resources/lib/PySide6/lupdate",
        f"{app_path}/Contents/Resources/lib/PySide6/Qt/qml",
        f"{app_path}/Contents/Resources/lib/PySide6/Assistant.app",
        f"{app_path}/Contents/Resources/lib/PySide6/Linguist.app",
        f"{app_path}/Contents/Resources/lib/PySide6/lrelease",
        f"{app_path}/Contents/Resources/lib/PySide6/qmlformat",
        f"{app_path}/Contents/Resources/lib/PySide6/qmllint",
        f"{app_path}/Contents/Resources/lib/PySide6/qmlls",
        f"{app_path}/Contents/Resources/lib/QtBluetooth",
        f"{app_path}/Contents/Resources/lib/QtConcurrent",
        f"{app_path}/Contents/Resources/lib/QtDesigner",
        f"{app_path}/Contents/Resources/lib/QtNetworkAuth",
        f"{app_path}/Contents/Resources/lib/QtNfc",
        f"{app_path}/Contents/Resources/lib/QtOpenGL",
        f"{app_path}/Contents/Resources/lib/QtOpenGLWidgets",
        f"{app_path}/Contents/Resources/lib/QtPositioning",
        f"{app_path}/Contents/Resources/lib/QtQuick3D",
        f"{app_path}/Contents/Resources/lib/QtQuick3DRuntimeRender",
        f"{app_path}/Contents/Resources/lib/QtQuick3DUtils",
        f"{app_path}/Contents/Resources/lib/QtShaderTools",
        f"{app_path}/Contents/Resources/lib/QtStateMachine",
        f"{app_path}/Contents/Resources/lib/QtSvgWidgets",
        f"{app_path}/Contents/Resources/lib/QtWebChannel",
        f"{app_path}/Contents/Resources/lib/QtWebEngineCore",
        f"{app_path}/Contents/Resources/lib/QtWebEngineQuick",
        f"{app_path}/Contents/Resources/lib/QtXml",
    ]:
        if os.path.isfile(filename) or os.path.islink(filename):
            os.remove(filename)
            print(f"Deleted: {filename}")
        elif os.path.isdir(filename):
            shutil.rmtree(filename)
            print(f"Deleted: {filename}")
        else:
            print(f"Cannot delete, filename not found: {filename}")

    # Set a symlink for Qt's platforms and imageformats plugins, which seems to be necessary only on arm
    if platform.system() == "Darwin" and platform.processor() == "arm":
        platforms_target_path = os.path.join("..", "Resources", "lib", "PySide6", "Qt", "plugins", "platforms")
        platforms_symlink_path = os.path.join(app_path, "Contents", "MacOS", "platforms")

        os.symlink(platforms_target_path, platforms_symlink_path)

        imageformats_target_path = os.path.join("..", "Resources", "lib", "PySide6", "Qt", "plugins", "imageformats")
        imageformats_symlink_path = os.path.join(app_path, "Contents", "MacOS", "imageformats")
        os.symlink(imageformats_target_path, imageformats_symlink_path)


    after_size = get_size(f"{app_path}")
    freed_bytes = before_size - after_size
    freed_mb = int(freed_bytes / 1024 / 1024)
    print(f"> Freed {freed_mb} mb")


@main.command()
@click.argument("app_path")
def codesign(app_path):
    """Sign macOS binaries before packaging"""
    bin_universal, bin_silicon, bin_intel = get_binary_arches(app_path)
    binaries = bin_universal + bin_silicon + bin_intel + [app_path]

    for filename in binaries:
        sign(filename, entitlements_plist_path, identity_name_application)

    print(f"> Signed app bundle: {app_path}")


@main.command()
@click.argument("app_path")
def package(app_path):
    """Build the DMG package"""
    if not os.path.exists("/usr/local/bin/create-dmg") and not os.path.exists(
        "/opt/homebrew/bin/create-dmg"
    ):
        print("> Error: create-dmg is not installed")
        return

    print("> Create DMG")
    version_filename = f"{root}/cli/onionshare_cli/resources/version.txt"
    with open(version_filename) as f:
        version = f.read().strip()

    os.makedirs(f"{desktop_dir}/dist", exist_ok=True)
    dmg_path = f"{desktop_dir}/dist/OnionShare-{version}.dmg"
    run(
        [
            "create-dmg",
            "--volname",
            "OnionShare",
            "--volicon",
            f"{desktop_dir}/onionshare/resources/onionshare.icns",
            "--window-size",
            "400",
            "200",
            "--icon-size",
            "100",
            "--icon",
            "OnionShare.app",
            "100",
            "70",
            "--hide-extension",
            "OnionShare.app",
            "--app-drop-link",
            "300",
            "70",
            dmg_path,
            app_path,
            "--identity",
            identity_name_application,
        ]
    )

    print(f"> Finished building DMG: {dmg_path}")


if __name__ == "__main__":
    main()



import glob
import pytest
import os
import requests
import shutil
import sys
from datetime import datetime, timedelta

from PySide6 import QtCore, QtTest

from .gui_base_test import GuiBaseTest


class TestReceive(GuiBaseTest):
    # Shared test methods

    def upload_file(
        self, tab, file_to_upload, expected_basename, identical_files_at_once=False
    ):
        """Test that we can upload the file"""

        # Wait 2 seconds to make sure the filename, based on timestamp, isn't accidentally reused
        QtTest.QTest.qWait(2000, self.gui.qtapp)

        files = {"file[]": open(file_to_upload, "rb")}
        url = f"http://127.0.0.1:{tab.app.port}/upload"
        requests.post(url, files=files)
        if identical_files_at_once:
            # Send a duplicate upload to test for collisions
            requests.post(url, files=files)

        QtTest.QTest.qWait(1000, self.gui.qtapp)

        # Make sure the file is within the last 10 seconds worth of filenames
        exists = False
        now = datetime.now()
        for _ in range(10):
            date_dir = now.strftime("%Y-%m-%d")
            time_dir = now.strftime("%H%M%S")
            receive_mode_dir = os.path.join(
                tab.settings.get("receive", "data_dir"), date_dir, time_dir
            )
            # The directories have microseconds in the name, so we need
            # to use globbing against directory names containing the same
            # second in order to try to find the file.
            for path in glob.glob(receive_mode_dir + "*"):
                if os.path.exists(os.path.join(path, expected_basename)):
                    exists = True
                    break
            now = now - timedelta(seconds=1)

        self.assertTrue(exists)

    def upload_file_should_fail(self, tab):
        """Test that we can't upload the file when permissions are wrong, and expected content is shown"""
        QtTest.QTest.qWait(1000, self.gui.qtapp)

        files = {"file[]": open(self.tmpfile_test, "rb")}
        url = f"http://127.0.0.1:{tab.app.port}/upload"
        r = requests.post(url, files=files)

        def accept_dialog():
            window = tab.common.gui.qtapp.activeWindow()
            if window:
                window.close()

        QtCore.QTimer.singleShot(1000, accept_dialog)
        self.assertTrue("Error uploading, please inform the OnionShare user" in r.text)

    def submit_message(self, tab, message):
        """Test that we can submit a message"""

        # Wait 2 seconds to make sure the filename, based on timestamp, isn't accidentally reused
        QtTest.QTest.qWait(2000, self.gui.qtapp)

        url = f"http://127.0.0.1:{tab.app.port}/upload"
        requests.post(url, data={"text": message})

        QtTest.QTest.qWait(1000, self.gui.qtapp)

        # Make sure the file is within the last 10 seconds worth of filenames
        exists = False
        now = datetime.now()
        for _ in range(10):
            date_dir = now.strftime("%Y-%m-%d")
            time_dir = now.strftime("%H%M%S")
            expected_estimated_filename = os.path.join(
                tab.settings.get("receive", "data_dir"),
                date_dir,
                f"{time_dir}*-message.txt",
            )
            for path in glob.glob(expected_estimated_filename):
                if os.path.exists(path):
                    with open(path) as f:
                        assert f.read() == message

                    exists = True
                    break
            now = now - timedelta(seconds=1)

        self.assertTrue(exists)

    # 'Grouped' tests follow from here

    def run_all_receive_mode_setup_tests(self, tab):
        """Set up a share in Receive mode and start it"""
        self.history_is_not_visible(tab)
        self.click_toggle_history(tab)
        self.history_is_visible(tab)
        self.server_working_on_start_button_pressed(tab)
        self.server_status_indicator_says_starting(tab)
        self.server_is_started(tab)
        self.web_server_is_running(tab)
        self.url_description_shown(tab)
        self.url_instructions_shown(tab)
        self.url_shown(tab)
        self.have_copy_url_button(tab)
        self.have_show_url_qr_code_button(tab)
        self.client_auth_instructions_shown(tab)
        self.private_key_shown(tab)
        self.have_show_client_auth_qr_code_button(tab)
        self.server_status_indicator_says_started(tab)

    def run_all_receive_mode_tests(self, tab):
        """Submit files and messages in receive mode and stop the share"""
        self.run_all_receive_mode_setup_tests(tab)
        self.javascript_is_correct_mime_type(tab, "receive.js")
        self.upload_file(tab, self.tmpfile_test, "test.txt")
        self.history_widgets_present(tab)
        self.counter_incremented(tab, 1)
        self.upload_file(tab, self.tmpfile_test, "test.txt")
        self.counter_incremented(tab, 2)
        self.upload_file(tab, self.tmpfile_test2, "test2.txt")
        self.counter_incremented(tab, 3)
        self.upload_file(tab, self.tmpfile_test2, "test2.txt")
        self.counter_incremented(tab, 4)
        self.submit_message(tab, "onionshare is an interesting piece of software")
        self.counter_incremented(tab, 5)
        # Test uploading the same file twice at the same time, and make sure no collisions
        self.upload_file(tab, self.tmpfile_test, "test.txt", True)
        self.counter_incremented(tab, 7)
        self.history_indicator(tab, "2")
        self.server_is_stopped(tab)
        self.web_server_is_stopped(tab)
        self.server_status_indicator_says_closed(tab)
        self.server_working_on_start_button_pressed(tab)
        self.server_is_started(tab)
        self.history_indicator(tab, "2")

    def run_all_clear_all_button_tests(self, tab):
        """Test the Clear All history button"""
        self.run_all_receive_mode_setup_tests(tab)
        self.upload_file(tab, self.tmpfile_test, "test.txt")
        self.history_widgets_present(tab)
        self.clear_all_history_items(tab, 0)
        self.upload_file(tab, self.tmpfile_test, "test.txt")
        self.clear_all_history_items(tab, 2)

    def run_all_upload_non_writable_dir_tests(self, tab):
        """Test uploading a file when the data_dir is non-writable"""
        upload_dir = os.path.join(self.tmpdir.name, "OnionShare")
        shutil.rmtree(upload_dir, ignore_errors=True)
        os.makedirs(upload_dir, 0o700)

        # Set the upload dir setting
        tab.get_mode().data_dir_lineedit.setText(upload_dir)
        tab.settings.set("receive", "data_dir", upload_dir)

        self.run_all_receive_mode_setup_tests(tab)
        os.chmod(upload_dir, 0o400)
        self.upload_file_should_fail(tab)
        self.server_is_stopped(tab)
        self.web_server_is_stopped(tab)
        self.server_status_indicator_says_closed(tab)
        os.chmod(upload_dir, 0o700)

    # Tests

    def test_clear_all_button(self):
        """
        Clear all history items should work
        """
        tab = self.new_receive_tab()

        self.run_all_common_setup_tests()
        self.run_all_clear_all_button_tests(tab)

        self.close_all_tabs()

    def test_autostop_timer(self):
        """
        Test autostop timer
        """
        tab = self.new_receive_tab()
        tab.get_mode().mode_settings_widget.toggle_advanced_button.click()
        tab.get_mode().mode_settings_widget.autostop_timer_checkbox.click()

        self.run_all_common_setup_tests()
        self.run_all_receive_mode_setup_tests(tab)
        self.set_timeout(tab, 5)
        self.autostop_timer_widget_hidden(tab)
        self.server_timed_out(tab, 15000)
        self.web_server_is_stopped(tab)

        self.close_all_tabs()

    def test_upload(self):
        """
        Test uploading files
        """
        tab = self.new_receive_tab()

        self.run_all_common_setup_tests()
        self.run_all_receive_mode_tests(tab)

        self.close_all_tabs()

    @pytest.mark.skipif(sys.platform == "win32", reason="Windows doesn't have chmod")
    def test_upload_non_writable_dir(self):
        """
        Test uploading files to a non-writable directory
        """
        tab = self.new_receive_tab()

        self.run_all_upload_non_writable_dir_tests(tab)

        self.close_all_tabs()

    def test_public_upload(self):
        """
        Test uploading files in public mode
        """
        tab = self.new_receive_tab()
        tab.get_mode().mode_settings_widget.public_checkbox.click()

        self.run_all_common_setup_tests()
        self.run_all_receive_mode_tests(tab)

        self.close_all_tabs()

    @pytest.mark.skipif(sys.platform == "win32", reason="Windows doesn't have chmod")
    def test_public_upload_non_writable_dir(self):
        """
        Test uploading files to a non-writable directory in public mode
        """
        tab = self.new_receive_tab()
        tab.get_mode().mode_settings_widget.public_checkbox.click()

        self.run_all_upload_non_writable_dir_tests(tab)

        self.close_all_tabs()

    def test_405_page_returned_for_invalid_methods(self):
        """
        Our custom 405 page should return for invalid methods
        """
        tab = self.new_receive_tab()

        tab.get_mode().mode_settings_widget.public_checkbox.click()

        self.run_all_common_setup_tests()
        self.run_all_receive_mode_setup_tests(tab)
        self.upload_file(tab, self.tmpfile_test, "test.txt")
        url = f"http://127.0.0.1:{tab.app.port}/"
        self.hit_405(
            url,
            expected_resp="OnionShare: 405 Method Not Allowed",
            data={"foo": "bar"},
            methods=["put", "post", "delete", "options"],
        )

        self.server_is_stopped(tab)
        self.web_server_is_stopped(tab)
        self.server_status_indicator_says_closed(tab)
        self.close_all_tabs()

    def test_disable_files_prevents_file_upload(self):
        """
        Test that disable_files setting prevents file uploads (security fix)
        When disable_files is True, file uploads should be rejected and no files written to disk.
        """
        tab = self.new_receive_tab()

        # Enable disable_files setting
        tab.get_mode().mode_settings_widget.toggle_advanced_button.click()
        tab.get_mode().disable_files_checkbox.click()

        self.run_all_common_setup_tests()
        self.run_all_receive_mode_setup_tests(tab)

        # Try to upload a file - should fail
        files = {"file[]": open(self.tmpfile_test, "rb")}
        url = f"http://127.0.0.1:{tab.app.port}/upload-ajax"
        r = requests.post(url, files=files)

        # Verify error response
        self.assertIn("error_flashes", r.text)
        self.assertIn("File uploads are not allowed", r.text)

        # Verify no files were written to disk
        QtTest.QTest.qWait(1000, self.gui.qtapp)
        written_files = []
        now = datetime.now()
        for _ in range(10):
            date_dir = now.strftime("%Y-%m-%d")
            time_dir = now.strftime("%H%M%S")
            receive_mode_dir = os.path.join(
                tab.settings.get("receive", "data_dir"), date_dir, time_dir
            )
            for path in glob.glob(receive_mode_dir + "*"):
                for root, dirs, files in os.walk(path):
                    for f in files:
                        if not f.endswith("-message.txt"):
                            written_files.append(os.path.join(root, f))
            now = now - timedelta(seconds=1)

        self.assertEqual(
            len(written_files),
            0,
            f"No files should be written when disable_files is True, but found: {written_files}",
        )

        self.server_is_stopped(tab)
        self.web_server_is_stopped(tab)
        self.server_status_indicator_says_closed(tab)
        self.close_all_tabs()

    def test_disable_files_allows_text_message(self):
        """
        Test that disable_files setting still allows text messages
        """
        tab = self.new_receive_tab()

        # Enable disable_files setting
        tab.get_mode().mode_settings_widget.toggle_advanced_button.click()
        tab.get_mode().disable_files_checkbox.click()

        self.run_all_common_setup_tests()
        self.run_all_receive_mode_setup_tests(tab)

        # Submit a text message - should succeed
        self.submit_message(tab, "This is a test message")

        # Verify counter incremented (text message was accepted)
        self.counter_incremented(tab, 1)

        self.server_is_stopped(tab)
        self.web_server_is_stopped(tab)
        self.server_status_indicator_says_closed(tab)
        self.close_all_tabs()



#!/usr/bin/env python3
import subprocess
import tempfile
import json
import os


def main():
    tmp_dir = tempfile.TemporaryDirectory()
    flagsdir = os.path.join(tmp_dir.name, "flagsicon")
    subprocess.run(["git", "clone", "https://github.com/lipis/flag-icons.git", flagsdir])

    with open(
        os.path.join("onionshare", "resources", "countries", "en.json")
    ) as f:
        countries = list(json.loads(f.read()))

    os.makedirs(
        os.path.join(
            "onionshare",
            "resources",
            "images",
            "countries",
        ),
        exist_ok=True,
    )

    for country in countries:
        country = country.lower()
        if os.path.isfile(os.path.join(flagsdir, "flags", "4x3", f"{country}.svg")):
            src_filename = os.path.join(flagsdir, "flags", "4x3", f"{country}.svg")
            dest_filename = os.path.join(
                "onionshare",
                "resources",
                "images",
                "countries",
                f"{country}.png",
            )
            subprocess.run(
                [
                    "convert",
                    src_filename,
                    "-background",
                    "none",
                    "-density",
                    "100",
                    "-resize",
                    "64x",
                    dest_filename,
                ]
            )


if __name__ == "__main__":
    main()



import unittest
import os
import requests
import shutil
import tempfile
import secrets
import platform
import sys

from PySide6 import QtCore, QtTest, QtWidgets

from onionshare_cli.common import Common

from onionshare import Application, MainWindow, GuiCommon
from onionshare.tab.mode.share_mode import ShareMode
from onionshare.tab.mode.receive_mode import ReceiveMode
from onionshare.tab.mode.website_mode import WebsiteMode
from onionshare.tab.mode.chat_mode import ChatMode
from onionshare import strings


class GuiBaseTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        common = sys.onionshare_common
        qtapp = sys.onionshare_qtapp

        # Delete any old test data that might exist
        shutil.rmtree(common.build_data_dir(), ignore_errors=True)

        common.gui = GuiCommon(common, qtapp, local_only=True)
        cls.gui = MainWindow(common, filenames=None)
        cls.gui.qtapp = qtapp

        # Create some random files to test with
        cls.tmpdir = tempfile.TemporaryDirectory()
        cls.tmpfiles = []
        for _ in range(10):
            filename = os.path.join(cls.tmpdir.name, f"{secrets.token_hex(4)}.txt")
            with open(filename, "w") as file:
                file.write(secrets.token_hex(10))
            cls.tmpfiles.append(filename)

        # A file called "test.txt"
        cls.tmpfile_test = os.path.join(cls.tmpdir.name, "test.txt")
        with open(cls.tmpfile_test, "w") as file:
            file.write("onionshare")

        # A file called "test2.txt"
        cls.tmpfile_test2 = os.path.join(cls.tmpdir.name, "test2.txt")
        with open(cls.tmpfile_test2, "w") as file:
            file.write("onionshare2")

        # A file called "index.html"
        cls.tmpfile_index_html = os.path.join(cls.tmpdir.name, "index.html")
        with open(cls.tmpfile_index_html, "w") as file:
            file.write(
                "<html><body><p>This is a test website hosted by OnionShare</p></body></html>"
            )

        # A file called "test.html"
        cls.tmpfile_test_html = os.path.join(cls.tmpdir.name, "test.html")
        with open(cls.tmpfile_test_html, "w") as file:
            file.write(
                "<html><body><p>Sed ut perspiciatis unde omnis iste natus error sit voluptatem accusantium doloremque laudantium, totam rem aperiam, eaque ipsa quae ab illo inventore veritatis et quasi architecto beatae vitae dicta sunt explicabo. Nemo enim ipsam voluptatem quia voluptas sit aspernatur aut odit aut fugit, sed quia consequuntur magni dolores eos qui ratione voluptatem sequi nesciunt. Neque porro quisquam est, qui dolorem ipsum quia dolor sit amet, consectetur, adipisci velit, sed quia non numquam eius modi tempora incidunt ut labore et dolore magnam aliquam quaerat voluptatem. Ut enim ad minima veniam, quis nostrum exercitationem ullam corporis suscipit laboriosam, nisi ut aliquid ex ea commodi consequatur? Quis autem vel eum iure reprehenderit qui in ea voluptate velit esse quam nihil molestiae consequatur, vel illum qui dolorem eum fugiat quo voluptas nulla pariatur?</p></body></html>"
            )

        # A large file
        size = 1024 * 1024 * 155
        cls.tmpfile_large = os.path.join(cls.tmpdir.name, "large_file")
        with open(cls.tmpfile_large, "wb") as fout:
            fout.write(os.urandom(size))

    @classmethod
    def tearDownClass(cls):
        # Quit
        cls.gui.qtapp.clipboard().clear()
        QtCore.QTimer.singleShot(200, cls.gui.close_dialog.accept_button.click)
        cls.gui.close()

        cls.gui.cleanup()

    # Shared test methods

    def verify_new_tab(self, tab):
        # Make sure the new tab widget is showing, and no mode has been started
        QtTest.QTest.qWait(1000, self.gui.qtapp)
        self.assertTrue(tab.new_tab.isVisible())
        self.assertFalse(hasattr(tab, "share_mode"))
        self.assertFalse(hasattr(tab, "receive_mode"))
        self.assertFalse(hasattr(tab, "website_mode"))
        self.assertFalse(hasattr(tab, "chat_mode"))

    def new_share_tab(self):
        tab = self.gui.tabs.widget(0)
        self.verify_new_tab(tab)

        # Share files
        tab.share_button.click()
        self.assertFalse(tab.new_tab.isVisible())
        self.assertTrue(tab.share_mode.isVisible())

        return tab

    def new_share_tab_with_files(self):
        tab = self.new_share_tab()

        # Add files
        for filename in self.tmpfiles:
            tab.share_mode.server_status.file_selection.file_list.add_file(filename)

        return tab

    def new_receive_tab(self):
        tab = self.gui.tabs.widget(0)
        self.verify_new_tab(tab)

        # Receive files
        tab.receive_button.click()
        self.assertFalse(tab.new_tab.isVisible())
        self.assertTrue(tab.receive_mode.isVisible())

        return tab

    def new_website_tab(self):
        tab = self.gui.tabs.widget(0)
        self.verify_new_tab(tab)

        # Publish website
        tab.website_button.click()
        self.assertFalse(tab.new_tab.isVisible())
        self.assertTrue(tab.website_mode.isVisible())

        return tab

    def new_website_tab_with_files(self):
        tab = self.new_website_tab()

        # Add files
        for filename in self.tmpfiles:
            tab.website_mode.server_status.file_selection.file_list.add_file(filename)

        return tab

    def new_chat_tab(self):
        tab = self.gui.tabs.widget(0)
        self.verify_new_tab(tab)

        # Chat
        tab.chat_button.click()
        self.assertFalse(tab.new_tab.isVisible())
        self.assertTrue(tab.chat_mode.isVisible())

        return tab

    def close_all_tabs(self):
        for _ in range(self.gui.tabs.count()):
            tab = self.gui.tabs.widget(0)
            QtCore.QTimer.singleShot(200, tab.close_dialog.accept_button.click)
            self.gui.tabs.tabBar().tabButton(0, QtWidgets.QTabBar.RightSide).click()

    def gui_loaded(self):
        """Test that the GUI actually is shown"""
        self.assertTrue(self.gui.show)

    def window_title_seen(self):
        """Test that the window title is OnionShare"""
        self.assertEqual(self.gui.windowTitle(), "OnionShare")

    def server_status_bar_is_visible(self):
        """Test that the status bar is visible"""
        self.assertTrue(self.gui.status_bar.isVisible())

    def mode_settings_widget_is_visible(self, tab):
        """Test that the mode settings are visible"""
        self.assertTrue(tab.get_mode().mode_settings_widget.isVisible())

    def mode_settings_widget_is_hidden(self, tab):
        """Test that the mode settings are hidden when the server starts"""
        self.assertFalse(tab.get_mode().mode_settings_widget.isVisible())

    def click_toggle_history(self, tab):
        """Test that we can toggle Download or Upload history by clicking the toggle button"""
        currently_visible = tab.get_mode().history.isVisible()
        tab.get_mode().toggle_history.click()
        self.assertEqual(tab.get_mode().history.isVisible(), not currently_visible)

    def javascript_is_correct_mime_type(self, tab, file):
        """Test that the javascript file send.js is fetchable and that its MIME type is correct"""
        path = f"{tab.get_mode().web.static_url_path}/js/{file}"
        url = f"http://127.0.0.1:{tab.app.port}/{path}"
        r = requests.get(url)
        self.assertTrue(r.headers["Content-Type"].startswith("text/javascript;"))

    def history_indicator(self, tab, indicator_count="1"):
        """Test that we can make sure the history is toggled off, do an action, and the indicator works"""
        # Make sure history is toggled off
        if tab.get_mode().history.isVisible():
            tab.get_mode().toggle_history.click()
            self.assertFalse(tab.get_mode().history.isVisible())

        # Indicator should not be visible yet
        self.assertFalse(tab.get_mode().toggle_history.indicator_label.isVisible())

        if type(tab.get_mode()) == ReceiveMode:
            # Upload a file
            files = {"file[]": open(self.tmpfiles[0], "rb")}
            url = f"http://127.0.0.1:{tab.app.port}/upload"
            requests.post(url, files=files)
            QtTest.QTest.qWait(2000, self.gui.qtapp)

        if type(tab.get_mode()) == ShareMode:
            # Download files
            url = f"http://127.0.0.1:{tab.app.port}/download"
            requests.get(url)
            QtTest.QTest.qWait(2000, self.gui.qtapp)

        # Indicator should be visible, have a value of "1"
        self.assertTrue(tab.get_mode().toggle_history.indicator_label.isVisible())
        self.assertEqual(
            tab.get_mode().toggle_history.indicator_label.text(), indicator_count
        )

        # Toggle history back on, indicator should be hidden again
        tab.get_mode().toggle_history.click()
        self.assertFalse(tab.get_mode().toggle_history.indicator_label.isVisible())

    def history_is_not_visible(self, tab):
        """Test that the History section is not visible"""
        self.assertFalse(tab.get_mode().history.isVisible())

    def history_is_visible(self, tab):
        """Test that the History section is visible"""
        self.assertTrue(tab.get_mode().history.isVisible())

    def server_working_on_start_button_pressed(self, tab):
        """Test we can start the service"""
        # Should be in SERVER_WORKING state
        tab.get_mode().server_status.server_button.click()
        self.assertEqual(tab.get_mode().server_status.status, 1)

    def toggle_indicator_is_reset(self, tab):
        self.assertEqual(tab.get_mode().toggle_history.indicator_count, 0)
        self.assertFalse(tab.get_mode().toggle_history.indicator_label.isVisible())

    def server_status_indicator_says_starting(self, tab):
        """Test that the Server Status indicator shows we are Starting"""
        self.assertEqual(
            tab.get_mode().server_status_label.text(),
            strings._("gui_status_indicator_share_working"),
        )

    def server_status_indicator_says_scheduled(self, tab):
        """Test that the Server Status indicator shows we are Scheduled"""
        self.assertEqual(
            tab.get_mode().server_status_label.text(),
            strings._("gui_status_indicator_share_scheduled"),
        )

    def server_is_started(self, tab, startup_time=2000):
        """Test that the server has started"""
        QtTest.QTest.qWait(startup_time, self.gui.qtapp)
        # Should now be in SERVER_STARTED state
        self.assertEqual(tab.get_mode().server_status.status, 2)

    def web_server_is_running(self, tab):
        """Test that the web server has started"""
        try:
            requests.get(f"http://127.0.0.1:{tab.app.port}/")
            self.assertTrue(True)
        except requests.exceptions.ConnectionError:
            self.assertTrue(False)

    def add_button_visible(self, tab):
        """Test that the add button should be visible"""
        if platform.system() == "Darwin":
            self.assertTrue(
                tab.get_mode().server_status.file_selection.add_files_button.isVisible()
            )
            self.assertTrue(
                tab.get_mode().server_status.file_selection.add_folder_button.isVisible()
            )
        else:
            self.assertTrue(
                tab.get_mode().server_status.file_selection.add_button.isVisible()
            )

    def url_shown(self, tab):
        """Test that the URL is showing"""
        self.assertTrue(tab.get_mode().server_status.url.isVisible())

    def url_description_shown(self, tab):
        """Test that the URL label is showing"""
        self.assertTrue(tab.get_mode().server_status.url_description.isVisible())

    def url_instructions_shown(self, tab):
        """Test that the URL instructions for sharing are showing"""
        self.assertTrue(tab.get_mode().server_status.url_instructions.isVisible())

    def private_key_shown(self, tab):
        """Test that the Private Key is showing when not in public mode"""
        if not tab.settings.get("general", "public"):
            # Both the private key field and the toggle button should be seen
            self.assertTrue(tab.get_mode().server_status.private_key.isVisible())
            self.assertTrue(
                tab.get_mode().server_status.client_auth_toggle_button.isVisible()
            )
            self.assertEqual(
                tab.get_mode().server_status.client_auth_toggle_button.text(),
                strings._("gui_reveal"),
            )

            # Test that the key is masked unless Reveal is clicked
            self.assertEqual(
                tab.get_mode().server_status.private_key.text(),
                "*" * len(tab.app.auth_string),
            )

            # Click reveal
            tab.get_mode().server_status.client_auth_toggle_button.click()

            # The real private key should be revealed
            self.assertEqual(
                tab.get_mode().server_status.private_key.text(), tab.app.auth_string
            )
            self.assertEqual(
                tab.get_mode().server_status.client_auth_toggle_button.text(),
                strings._("gui_hide"),
            )

            # Click hide, key should be masked again
            tab.get_mode().server_status.client_auth_toggle_button.click()
            self.assertEqual(
                tab.get_mode().server_status.private_key.text(),
                "*" * len(tab.app.auth_string),
            )
            self.assertEqual(
                tab.get_mode().server_status.client_auth_toggle_button.text(),
                strings._("gui_reveal"),
            )
        else:
            self.assertFalse(tab.get_mode().server_status.private_key.isVisible())

    def client_auth_instructions_shown(self, tab):
        """
        Test that the Private Key instructions for sharing
        are showing when not in public mode
        """
        if not tab.settings.get("general", "public"):
            self.assertTrue(
                tab.get_mode().server_status.client_auth_instructions.isVisible()
            )
        else:
            self.assertFalse(
                tab.get_mode().server_status.client_auth_instructions.isVisible()
            )

    def have_copy_url_button(self, tab):
        """Test that the Copy URL button is shown and that the clipboard is correct"""
        self.assertTrue(tab.get_mode().server_status.copy_url_button.isVisible())

        tab.get_mode().server_status.copy_url_button.click()
        clipboard = tab.common.gui.qtapp.clipboard()
        self.assertEqual(clipboard.text(), f"http://127.0.0.1:{tab.app.port}")

    def have_show_url_qr_code_button(self, tab):
        """Test that the Show QR Code URL button is shown and that it loads a QR Code Dialog"""
        self.assertTrue(
            tab.get_mode().server_status.show_url_qr_code_button.isVisible()
        )

        def accept_dialog():
            window = tab.common.gui.qtapp.activeWindow()
            if window:
                window.close()

        QtCore.QTimer.singleShot(500, accept_dialog)
        tab.get_mode().server_status.show_url_qr_code_button.click()

    def have_show_client_auth_qr_code_button(self, tab):
        """
        Test that the Show QR Code Client Auth button is shown when
        not in public mode and that it loads a QR Code Dialog.
        """
        if not tab.settings.get("general", "public"):
            self.assertTrue(
                tab.get_mode().server_status.show_client_auth_qr_code_button.isVisible()
            )

            def accept_dialog():
                window = tab.common.gui.qtapp.activeWindow()
                if window:
                    window.close()

            QtCore.QTimer.singleShot(500, accept_dialog)
            tab.get_mode().server_status.show_client_auth_qr_code_button.click()
        else:
            self.assertFalse(
                tab.get_mode().server_status.show_client_auth_qr_code_button.isVisible()
            )

    def server_status_indicator_says_started(self, tab):
        """Test that the Server Status indicator shows we are started"""
        if type(tab.get_mode()) == ReceiveMode:
            self.assertEqual(
                tab.get_mode().server_status_label.text(),
                strings._("gui_status_indicator_receive_started"),
            )
        if type(tab.get_mode()) == ShareMode:
            self.assertEqual(
                tab.get_mode().server_status_label.text(),
                strings._("gui_status_indicator_share_started"),
            )

    def web_page(self, tab, string):
        """Test that the web page contains a string"""

        url = f"http://127.0.0.1:{tab.app.port}/"
        r = requests.get(url)
        self.assertTrue(string in r.text)

    def history_widgets_present(self, tab):
        """Test that the relevant widgets are present in the history view after activity has taken place"""
        self.assertFalse(tab.get_mode().history.empty.isVisible())
        self.assertTrue(tab.get_mode().history.not_empty.isVisible())

    def counter_incremented(self, tab, count):
        """Test that the counter has incremented"""
        self.assertEqual(tab.get_mode().history.completed_count, count)

    def server_is_stopped(self, tab):
        """Test that the server stops when we click Stop"""
        if (
            type(tab.get_mode()) == ReceiveMode
            or (
                type(tab.get_mode()) == ShareMode
                and not tab.settings.get("share", "autostop_sharing")
            )
            or (type(tab.get_mode()) == WebsiteMode)
            or (type(tab.get_mode()) == ChatMode)
        ):
            tab.get_mode().server_status.server_button.click()
        self.assertEqual(tab.get_mode().server_status.status, 0)
        self.assertFalse(
            tab.get_mode().server_status.show_url_qr_code_button.isVisible()
        )
        self.assertFalse(tab.get_mode().server_status.copy_url_button.isVisible())
        self.assertFalse(tab.get_mode().server_status.url.isVisible())
        self.assertFalse(tab.get_mode().server_status.url_description.isVisible())
        self.assertFalse(tab.get_mode().server_status.url_instructions.isVisible())
        self.assertFalse(tab.get_mode().server_status.private_key.isVisible())
        self.assertFalse(
            tab.get_mode().server_status.client_auth_instructions.isVisible()
        )
        self.assertFalse(
            tab.get_mode().server_status.copy_client_auth_button.isVisible()
        )

    def web_server_is_stopped(self, tab):
        """Test that the web server also stopped"""
        QtTest.QTest.qWait(800, self.gui.qtapp)

        try:
            requests.get(f"http://127.0.0.1:{tab.app.port}/")
            self.assertTrue(False)
        except requests.exceptions.ConnectionError:
            self.assertTrue(True)

    def server_status_indicator_says_closed(self, tab):
        """Test that the Server Status indicator shows we closed"""
        if type(tab.get_mode()) == ReceiveMode:
            self.assertEqual(
                tab.get_mode().server_status_label.text(),
                strings._("gui_status_indicator_receive_stopped"),
            )
        if type(tab.get_mode()) == ShareMode:
            if not tab.settings.get("share", "autostop_sharing"):
                self.assertEqual(
                    tab.get_mode().server_status_label.text(),
                    strings._("gui_status_indicator_share_stopped"),
                )
            else:
                self.assertEqual(
                    tab.get_mode().server_status_label.text(),
                    strings._("closing_automatically"),
                )

    def clear_all_history_items(self, tab, count):
        if count == 0:
            tab.get_mode().history.clear_button.click()
        self.assertEqual(len(tab.get_mode().history.item_list.items.keys()), count)

    def file_selection_widget_has_files(self, tab, num=3):
        """Test that the number of items in the list is as expected"""
        self.assertEqual(
            tab.get_mode().server_status.file_selection.get_num_files(), num
        )

    def add_remove_buttons_hidden(self, tab):
        """Test that the add and remove buttons are hidden when the server starts"""
        if platform.system() == "Darwin":
            self.assertFalse(
                tab.get_mode().server_status.file_selection.add_files_button.isVisible()
            )
            self.assertFalse(
                tab.get_mode().server_status.file_selection.add_folder_button.isVisible()
            )
        else:
            self.assertFalse(
                tab.get_mode().server_status.file_selection.add_button.isVisible()
            )
        self.assertFalse(
            tab.get_mode().server_status.file_selection.remove_button.isVisible()
        )

    # Auto-stop timer tests
    def set_timeout(self, tab, timeout):
        """Test that the timeout can be set"""
        timer = QtCore.QDateTime.currentDateTime().addSecs(timeout)
        tab.get_mode().mode_settings_widget.autostop_timer_widget.setDateTime(timer)
        self.assertTrue(
            tab.get_mode().mode_settings_widget.autostop_timer_widget.dateTime(), timer
        )

    def autostop_timer_widget_hidden(self, tab):
        """Test that the auto-stop timer widget is hidden when share has started"""
        self.assertFalse(
            tab.get_mode().mode_settings_widget.autostop_timer_widget.isVisible()
        )

    def server_timed_out(self, tab, wait):
        """Test that the server has timed out after the timer ran out"""
        QtTest.QTest.qWait(wait, self.gui.qtapp)
        # We should have timed out now
        self.assertEqual(tab.get_mode().server_status.status, 0)

    def clientauth_is_visible(self, tab):
        """Test that the ClientAuth button is visible and that the clipboard contains its contents"""
        self.assertTrue(
            tab.get_mode().server_status.copy_client_auth_button.isVisible()
        )
        tab.get_mode().server_status.copy_client_auth_button.click()
        clipboard = tab.common.gui.qtapp.clipboard()
        self.assertEqual(
            clipboard.text(), "E2GOT5LTUTP3OAMRCRXO4GSH6VKJEUOXZQUC336SRKAHTTT5OVSA"
        )

    def clientauth_is_not_visible(self, tab):
        """Test that the ClientAuth button is not visible"""
        self.assertFalse(
            tab.get_mode().server_status.copy_client_auth_button.isVisible()
        )

    def hit_405(self, url, expected_resp, data={}, methods=[]):
        """Test various HTTP methods and the response"""
        for method in methods:
            if method == "put":
                r = requests.put(url, data=data)
            if method == "post":
                r = requests.post(url, data=data)
            if method == "delete":
                r = requests.delete(url)
            if method == "options":
                r = requests.options(url)
            self.assertTrue(expected_resp in r.text)
            self.assertFalse("Werkzeug" in r.headers)

    # Grouped tests follow from here

    def run_all_common_setup_tests(self):
        self.gui_loaded()
        self.window_title_seen()
        self.server_status_bar_is_visible()
